	.XLIST
USERAM	EQU	NEIN		; Ramtreiber bleibt unbenutzt

;  |----------------+--------------------------------------------|
;  | Funktion:      | Ansteuerprogramm fuer I2C Bus Peripherie   |
;  |                | auf PRINT/CTC Karte.                       |
;  |----------------+--------------------------------------------|
;  | Aufruf:        | Unterprogramme CLKINI, CLKSET, CLKGET, RAM |
;  |                | SET, RAMGET. Bei der Initialisierung kann  |
;  |                | ein Vektor zum Bearbeiten des 128 Hz in-   |
;  |                | terrupt uebergeben weren.                  |
;  |----------------+--------------------------------------------|
;  | Besonderheiten:| Hardwarespezifisches Programm fuer ELZET80 |
;  |                | PIO/CTC Karte. Achtung, I2CINI fuehrt auf  |
;  |                | PIO Kanal A keine Initialisierung durch.   |
;  |----------------+--------------------------------------------|
;  | History:       | 22.10.84  Start der Codierung.             |
;  |----------------+--------------------------------------------|
;  | Revision:      | 1.0 vom 22.10 1984                         |
;  |----------------+--------------------------------------------|
;  | Programmierer: | Michael Bleistein                          |
;  +-------------------------------------------------------------+
;
;  Es wurden einige Verkuerzungen vorgenommen ohne den Programm-
;  ablauf zu Aendern. Aenderungen in Kleinbuchstaben 
;	R.O.Koerber 851107
;

                TITLE   'CLOCK.ASM Uhrentreiber ELZET80 PIO/CTC'


;  +-------------------------------------------------------------+
;  | Konstantenvereinbarungen                                    |
;  |----------------+--------------------------------------------|
;  |                |                                            |
;  |                |                                            |
;  +-------------------------------------------------------------+

BASE_ADR        EQU     020H

                ; Adressen der Peripheriebausteine

CTC_0           EQU     BASE_ADR+0
CTC_1           EQU     BASE_ADR+1
CTC_2           EQU     BASE_ADR+2
CTC_3           EQU     BASE_ADR+3

PIO_AD          EQU     BASE_ADR+4
PIO_BD          EQU     BASE_ADR+5
PIO_AC          EQU     BASE_ADR+6
PIO_BC          EQU     BASE_ADR+7

                ; Bitbelegung I2C Bus

CLOCK_BIT       EQU	7;6
DATA_BIT	EQU	6;7

                ; Steuerworte PIO/CTC

BITMODE		EQU	11001111B
CTCRESET	EQU	00000011B
CTCMODE		EQU	11010111B

                ; Adressbytes I2C Bus

I2CCLK          EQU	11010000B
I2CRAM          EQU	10100000B

		; Subadressbytes I2C Bus

HOUR		EQU	00000000B
MINUTE		EQU	00000001B
DAY		EQU	00000010B
MONTH		EQU	00000011B
 
		
;  +-------------------------------------------------------------+
;  | Globale Daten                                               |
;  |----------------+--------------------------------------------|

                PUBLIC  I2CINI
                PUBLIC  CLKSET
                PUBLIC  CLKGET
   if	useram
                PUBLIC  RAMSET
                PUBLIC  RAMGET
   endif


;  +-------------------------------------------------------------+
;  | Unterprogramm: | UTILITY Funktionen I2C Bus                 |
;  |----------------+--------------------------------------------|
;  | Funktion:      | Durchfuehren primitiver Operationen auf    |
;  |                | dem I2C Bus.                               |
;  |----------------+--------------------------------------------|
;  | Input:         |                                            |
;  |                |                                            |
;  |----------------+--------------------------------------------|
;  | Output:        |                                            |
;  |                |                                            |
;  |----------------+--------------------------------------------|
;  | Register:      |                                            |
;  |                |                                            |
;  +-------------------------------------------------------------+


		;  +---------------------------------------------+
		;  | Unterprogramm: | CLK/DAT 0/1                |
		;  |----------------+----------------------------|
		;  | Funktion:      | Setzen der CLK oder DAT    |
		;  |                | Signale auf entsprechenden |
                ;  |                | Pegel.                     |
		;  +---------------------------------------------+

CLK0:		PUSH	AF
		IN	A,(PIO_AD)
		RES	CLOCK_BIT,A
		OUT	(PIO_AD),A
		LD	A,BITMODE
		OUT	(PIO_AC),A
		LD	A,(CURIO)
		RES	CLOCK_BIT,A
		LD	(CURIO),A
		OUT	(PIO_AC),A
		POP	AF
		RET


CLK1:		PUSH	AF
		LD	A,BITMODE
		OUT	(PIO_AC),A
		LD	A,(CURIO)
		SET	CLOCK_BIT,A
		LD	(CURIO),A
		OUT	(PIO_AC),A
CLK1_0:		IN	A,(PIO_AD)
		BIT	CLOCK_BIT,A
		JR	Z,CLK1_0
		POP	AF
		RET


DAT0:		PUSH	AF
		IN	A,(PIO_AD)
		RES	DATA_BIT,A
		OUT	(PIO_AD),A
		LD	A,BITMODE
		OUT	(PIO_AC),A
		LD	A,(CURIO)
		RES	DATA_BIT,A
		LD	(CURIO),A
		OUT	(PIO_AC),A
		POP	AF
		RET


DAT1:		PUSH	AF
		LD	A,BITMODE
		OUT	(PIO_AC),A
		LD	A,(CURIO)
		SET	DATA_BIT,A
		LD	(CURIO),A
		OUT	(PIO_AC),A
DAT1_0:		IN	A,(PIO_AD)
		BIT	DATA_BIT,A
		JR	Z,DAT1_0
		POP	AF
		RET


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CSTA                     |
		;  |----------------+----------------------------|
		;  | Funktion:      | Generiere Startbedingung   |
		;  |                | auf I2C Bus.               |
		;  +---------------------------------------------+

I2CSTA:		CALL	DAT1
		CALL	CLK1
		CALL	DAT0
		jp	CLK0
	

		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CSTO                     |
		;  |----------------+----------------------------|
		;  | Funktion:      | Generiere Stopbedingung    |
		;  |                | auf I2C Bus.               |
		;  +---------------------------------------------+

I2CSTO:		CALL	DAT0
		CALL	CLK1
		jp	DAT1


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CSND                     |
		;  |----------------+----------------------------|
		;  | Funktion:      | Sende das im A-Register    |
		;  |                | uebergebene Datenbyte.     |
		;  +---------------------------------------------+

I2CSND:		PUSH	BC
		LD	B,8
I2CSND_1:	RLCA
		JP	C,I2CSND_0
		CALL	DAT0
		JR	I2CSND_2
I2CSND_0:	CALL	DAT1
I2CSND_2:	CALL	CLK1
		CALL	CLK0
		DJNZ	I2CSND_1
		POP	BC
		RET


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CRCV                     |
		;  |----------------+----------------------------|
		;  | Funktion:      | Empfange ein Datenbyte vom |
		;  |                | I2C Bus im A-Register.     |
		;  +---------------------------------------------+

I2CRCV:		PUSH	BC
		LD	B,8
		LD	A,BITMODE
		OUT	(PIO_AC),A
		LD	A,(CURIO)
		SET	DATA_BIT,A
		LD	(CURIO),A
		OUT	(PIO_AC),A
I2CRCV_0:	CALL	CLK1
		IN	A,(PIO_AD)
		BIT	DATA_BIT,A
		SCF
		JR	NZ,I2CRCV_1
		CCF
I2CRCV_1:	RL	C
		CALL	CLK0
		DJNZ	I2CRCV_0
		LD	A,C
		POP	BC
		RET


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CSAK                     |
		;  |----------------+----------------------------|
		;  | Funktion:      | Sende die ACK Bedingung    |
		;  |                | auf den I2C Bus.           |
		;  +---------------------------------------------+

I2CSAK:		CALL	DAT0
		CALL	CLK1
		jp	CLK0


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CNAK                     |
		;  |----------------+----------------------------|
		;  | Funktion:      | Sende die I2C Bus NAK Be-  |
		;  |                | dingung.                   |
		;  +---------------------------------------------+

I2CSNA:		CALL	DAT1
		CALL	CLK1
		jp	CLK0


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CGAK                     |
		;  |----------------+----------------------------|
		;  | Funktion:      | Lese das ACK Signal vom    |
		;  |                | I2C Bus. Der Buspegel wird |
                ;  |                | im A-Register (0 = 00H und |
                ;  |                | 1 = 0FFH) uebergeben.      |
		;  +---------------------------------------------+


I2CGAK:		LD	A,BITMODE
		OUT	(PIO_AC),A
		LD	A,(CURIO)
		SET	DATA_BIT,A
		LD	(CURIO),A
		OUT	(PIO_AC),A
		CALL	CLK1
		IN	A,(PIO_AD)
		CALL	CLK0
		BIT	DATA_BIT,A
		LD	A,0
		RET	Z
		DEC	A
		RET


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CWR                      |
		;  |----------------+----------------------------|
		;  | Funktion:      | Der I2C Bus wird auf der   |
		;  |                | Adresse H, Subadresse L    |
                ;  |                | angesprochen und der In-   |
                ;  |                | halt des A-Registers aus-  |
                ;  |                | gegeben.                   |
		;  +---------------------------------------------+

I2CWR:		PUSH	BC
		PUSH	DE
		LD	B,A
		CALL	I2CSTA
		LD	A,H
		AND	11111110B
		CALL	I2CSND
		CALL	I2CGAK
		AND	A
		JP	NZ,I2CERR
		LD	A,L
		CALL	I2CSND
		CALL	I2CGAK
		AND	A
		JP	NZ,I2CERR
		LD	A,B
		CALL	I2CSND
		CALL	I2CGAK
		AND	A
		JR	NZ,I2CERR
		CALL	I2CSTO
		POP	DE
		POP	BC
		RET


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CRD                      |
		;  |----------------+----------------------------|
		;  | Funktion:      | Der I2C Bus wird auf der   |
		;  |                | Adresse H, Subadresse L    |
                ;  |                | angesprochen. Der eingele- |
                ;  |                | sene Wert wird im A-Re-    |
                ;  |                | gister uebergeben.         |
		;  +---------------------------------------------+

I2CRD:		PUSH	BC
		PUSH	DE
		CALL	I2CSTA
		LD	A,H
		AND	11111110B
		CALL	I2CSND
		CALL	I2CGAK
		AND	A
		JR	NZ,I2CERR
		LD	A,L
		CALL	I2CSND
		CALL	I2CGAK
		AND	A
		JR	NZ,I2CERR
		CALL	I2CSTA
		LD	A,H
		OR	00000001B
		CALL	I2CSND
		CALL	I2CGAK
		AND	A
		JR	NZ,I2CERR
		CALL	I2CRCV
		PUSH	AF
		CALL	I2CSNA
		CALL	I2CSTO
		POP	AF
		POP	DE
		POP	BC
		RET


		;  +---------------------------------------------+
		;  | Unterprogramm: | I2CERR                     |
		;  |----------------+----------------------------|
		;  | Funktion:      | Der I2C Buszyklus wird ab- |
		;  |                | gebrochen und das A-Re-    |
                ;  |                | gister auf 0 gesetzt.      |
		;  +---------------------------------------------+

I2CERR:		CALL	I2CSTO
		POP	DE
		POP	BC
		XOR	A
		RET


;  +-------------------------------------------------------------+
;  | Unterprogramm: | CLKSET                                     |
;  |----------------+--------------------------------------------|
;  | Funktion:      | Setzen der Echtzeituhr auf das uebergebene |
;  |                | Datum und die uebergebene Uhrzeit.         |
;  |----------------+--------------------------------------------|
;  | Input:         | TOS  Zeiger auf Datums- und Uhrzeitfeld.   |
;  |                |      In diesem Feld sind die einzelnen     |
;  |                |      Felder BCD codiert. Je 2 BCD Ziffern  |
;  |                |      sind in einem Byte angeordnet.        |
;  |                |                                            |
;  |                |        MM/DD/HH/MM                         |
;  |                |                                            |
;  |                |      (Monat, Tag, Stunde und Minute)       |
;  |----------------+--------------------------------------------|
;  | Output:        |                                            |
;  |----------------+--------------------------------------------|
;  | Register:      | Alle Registerinhalte ausser AF werden ge-  |
;  |                | rettet.                                    |
;  +-------------------------------------------------------------+

CLKSET:		PUSH	BC
		PUSH	DE
		PUSH	HL
		LD	HL,8
		ADD	HL,SP
		LD	E,(HL)
		INC	HL
		LD	D,(HL)

		; Schreibe Monat

		LD	A,(DE)
		INC	DE
		LD	HL,MONTH + (I2CCLK SHL 8)
		CALL	I2CWR

		; Schreibe Tag

		LD	A,(DE)
		INC	DE
		LD	HL,DAY + (I2CCLK SHL 8)
		CALL	I2CWR

		; Schreibe Stunde

		LD	A,(DE)
		INC	DE
		LD	HL,HOUR + (I2CCLK SHL 8)
		CALL	I2CWR

		; Schreibe Minute

		LD	A,(DE)
		INC	DE
		LD	HL,MINUTE + (I2CCLK SHL 8)
		CALL	I2CWR

		; Unterprogrammaussprung

		POP	HL
		POP	DE
		POP	BC
		RET     


;  +-------------------------------------------------------------+
;  | Unterprogramm: | CLKGET                                     |
;  |----------------+--------------------------------------------|
;  | Funktion:      | Auslesen der Echtzeituhr in den ueberge-   |
;  |                | benen Datums- und Uhrzeitblock.  .         |
;  |----------------+--------------------------------------------|
;  | Input:         | TOS  Zeiger auf Parameterblock.            |
;  |----------------+--------------------------------------------|
;  | Output:        | TOS  Zeiger auf Datums- und Uhrzeitfeld.   |
;  |                |      In diesem Feld sind die einzelnen     |
;  |                |      Felder BCD codiert. Je 2 BCD Ziffern  |
;  |                |      sind in einem Byte angeordnet.        |
;  |                |                                            |
;  |                |        MM/DD/HH/MM                         |
;  |                |                                            |
;  |                |      (Monat, Tag, Stunde und Minute)       |
;  |----------------+--------------------------------------------|
;  | Register:      | Alle Registerinhalte ausser AF werden ge-  |
;  |                | rettet.                                    |
;  +-------------------------------------------------------------+

CLKGET:		PUSH	BC
		PUSH	DE
		PUSH	HL
		LD	HL,8
		ADD	HL,SP
		LD	E,(HL)
		INC	HL
		LD	D,(HL)

		; Lese Monat

		LD	HL,MONTH + (I2CCLK SHL 8)
		CALL	I2CRD
		LD	(DE),A
		INC	DE	

		; Lese Tag

		LD	HL,DAY + (I2CCLK SHL 8)
		CALL	I2CRD
		LD	(DE),A
		INC	DE	

		; Lese Stunde

		LD	HL,HOUR + (I2CCLK SHL 8)
		CALL	I2CRD
		LD	(DE),A
		INC	DE	

		; Lese Minute

		LD	HL,MINUTE + (I2CCLK SHL 8)
		CALL	I2CRD
		LD	(DE),A
		INC	DE	

		; Unterprogrammaussprung

		POP	HL
		POP	DE
		POP	BC
		RET     

   if	useram			; nur wenn benutzt einbinden

;  +-------------------------------------------------------------+
;  | Unterprogramm: | RAMSET                                     |
;  |----------------+--------------------------------------------|
;  | Funktion:      | Setzen des akkugepufferten RAM Bausteins   |
;  |                | mit einer Bytesequenz.                     |
;  |----------------+--------------------------------------------|
;  | Input:         | TOS  Zeiger auf Parameterblock. In den er- |
;  |                |      sten beiden Bytes sind Startadresse   |
;  |                |      und Laenge im I2C Ram definiert. Es   |
;  |                |      folgt die festgelegte Anzahl an Da-   |
;  |                |      tenbytes.                             |
;  |----------------+--------------------------------------------|
;  | Output:        |                                            |
;  |----------------+--------------------------------------------|
;  | Register:      | Alle Registerinhalte ausser AF werden ge-  |
;  |                | rettet.                                    |
;  +-------------------------------------------------------------+

RAMSET:		PUSH	BC
		PUSH	DE
		PUSH	HL
		LD	HL,8
		ADD	HL,SP
		LD	E,(HL)
		INC	HL
		LD	D,(HL)

                ; Lese Startadresse

		LD	A,(DE)
		LD	C,A
		INC	DE

                ; Lese Bytelaenge

		LD	A,(DE)
		LD	B,A
		INC	DE

                ; Schreibe I2C Ram

		LD	H,I2CRAM
		LD	L,C
RAMSET_0:	LD	A,(DE)
		INC	DE
		CALL	I2CWR
		INC	L
		DJNZ	RAMSET_0

		; Unterprogrammaussprung

		POP	HL
		POP	DE
		POP	BC
		RET     


;  +-------------------------------------------------------------+
;  | Unterprogramm: | RAMGET                                     |
;  |----------------+--------------------------------------------|
;  | Funktion:      | Auslesen des akkugepufferten I2C Rams in   |
;  |                | den Parameterblock.  .                     |
;  |----------------+--------------------------------------------|
;  | Input:         | TOS  Zeiger auf Parameterblock. In den er- |
;  |                |      sten beiden Bytes sind Startadresse   |
;  |                |      und Laenge im I2C Ram definiert. Es   |
;  |                |      folgt der Speicherplatz zum Auslesen  |
;  |                |      des I2C Rams.                         |
;  |----------------+--------------------------------------------|
;  | Output:        | TOS+2  Ausgelesene Datenbytes des I2C Ram. |
;  |----------------+--------------------------------------------|
;  | Register:      | Alle Registerinhalte ausser AF werden ge-  |
;  |                | rettet.                                    |
;  +-------------------------------------------------------------+

RAMGET:		PUSH	BC
		PUSH	DE
		PUSH	HL
		LD	HL,8
		ADD	HL,SP
		LD	E,(HL)
		INC	HL
		LD	D,(HL)

                ; Lese Startadresse

		LD	A,(DE)
		LD	C,A
		INC	DE

                ; Lese Bytelaenge

		LD	A,(DE)
		LD	B,A
		INC	DE

                ; Lese I2C Ram

		LD	H,I2CRAM
		LD	L,C
RAMGET_0:	CALL	I2CRD
		LD	(DE),A
		INC	DE
		INC	L
		DJNZ	RAMGET_0

		; Unterprogrammaussprung

		POP	HL
		POP	DE
		POP	BC
		RET     

   endif

;  +-------------------------------------------------------------+
;  | Unterprogramm: | I2CINI                                     |
;  |----------------+--------------------------------------------|
;  | Funktion:      | Initialisieren des I2C Bus Softwareinter-  |
;  |                | face. Achtung, Die PIO der PIO/CTC Karte   |
;  |                | wird nicht initialisiert um Konflikte mit  |
;  |                | Druckeransteuerungen etc. zu vermeiden.    |
;  |----------------+--------------------------------------------|
;  | Input:         | TOS  Zeiger auf Parameterblock. Im Para-   |
;  |                |      meterblock sind die folgenden Daten   |
;  |                |      abgelegt:                             |
;  |                |                                            |
;  |                |       INT  Interrupt Nummer fuer Echtzeit- |
;  |                |            interrupt (== Position des Vek- |
;  |                |            tors in der IM2 Tabelle).       |
;  |                |       CNT  Teilerfaktor fuer CTC. Ist der  |
;  |                |            Wert = 0, so wird der CTC de-   |
;  |                |            aktiviert.                      |
;  |                |       DEF  Grundbelegung der Datenrichtung |
;  |                |            fuer PIO Kanal A (1 = in, 0 =   |
;  |                |            out).                           |
;  |                |                                            |
;  |                |     Alle Werte sind 1 Byte lang.           |
;  |----------------+--------------------------------------------|
;  | Output:        |                                            |
;  |----------------+--------------------------------------------|
;  | Register:      | Alle Registerinhalte werden bewahrt.       |
;  +-------------------------------------------------------------+

I2CINI:		PUSH	BC
		PUSH	DE
		PUSH	HL
		LD	HL,8
		ADD	HL,SP
		LD	A,(HL)
		INC	HL
		LD	H,(HL)
		LD	L,A

		; E-Register = Interruptnummer

		LD	E,(HL)
		INC	HL

		; D-Register = Zaehlerwert

		LD	D,(HL)
		INC	HL
		LD	A,(HL)
		LD	(CURIO),A
		LD	A,D
		AND	A
		JR	Z,I2CINI_0

		; Initialisiere CTC

		LD	A,CTCRESET
		OUT	(CTC_0),A
		LD	A,E
		OUT	(CTC_0),A
		LD	A,CTCMODE
		OUT	(CTC_0),A
		LD	A,D
		OUT	(CTC_0),A
		JR	I2CINI_1

		; Reset CTC

I2CINI_0:	LD	A,CTCRESET
		OUT	(CTC_0),A

		; Initialisiere PIO

I2CINI_1:	LD	A,BITMODE
		OUT	(PIO_AC),A
		LD	A,(CURIO)
		SET	DATA_BIT,A
		SET	CLOCK_BIT,A
		LD	(CURIO),A
		OUT	(PIO_AC),A

		; Unterprogrammende

		POP	HL
		POP	DE
		POP	BC
		RET     


;  +-------------------------------------------------------------+
;  | Datenbereich                                                |
;  +-------------------------------------------------------------+


CURIO:		DEFS	1


;  +-------------------------------------------------------------+
;  | Programmende                                                |
;  +-------------------------------------------------------------+

	.LIST
