; ****************************************************************
; *								 *
; * MODUL	STATLINE.INC					 *
; * REL		2.1			860212			 *
; *								 *
; * Dieses Programm-Modul ermoeglicht die Darstellung einer	 *
; * Statuszeile ( soweit technisch moeglich ) in der neben der   *
; * aktuellen Uhrzeit noch Diskettenname und Disketten-Rest-	 *
; * Kapazitaet eingeblendet wird				 *
; * Dieses Programm funktioniert nur falls ein Interrupt zur     *
; * Verfuegung steht.						 *
; *								 *	
; * geschrieben von RAOUL.O.KOERBER, Detmold			 *
; *								 *
; ****************************************************************

	;
	; ****************
	; * Statuszeile  *
	; ****************
	;
	; Die Ausgabe der Statuszeile ist eine etwas kritische Angelegenheit.
	; Sie darf keinesfalls innerhalb der Ausgabe einer ESC-Sequenz erfolgen
	; da sonst dieses Sequenz unterbrochen wuerde, was vor allem bei
	; Bildschirmorientierten Ausgaben zu unvorhersehbaren Ergebnissen
	; fuehren wurde. Am unkritischten ist die Ausgabe waehrend der CONIST-
	; Phase obwohl auch dann genau in die Eingabe einer ESC-Sequenz 'ge-
	; troffen' werden kann. Die meisten Editoren haben jedoch CNTRL-Zeichen
	; zur Cursorsteuerung so dass ESC-Sequenzen mehr aus- als eingegeben
	; werden. Sollten Stoerungen auftreten, so muss evtl. bei CONIN Flags
	; zur Erkennung von ESC-Sequenzen eingefuegt werden. Es kann dazu
	; das nachfolgende Programmsegment verwendet werden. (Aufzurufen NACH
	; CONIN)
	;
	; hltesc:	push 	af
	;		cp	esc	
	;		jr	nz,hlte2	; Wenn nicht	
	;		and	00000011b	; A=3 
	;		ld	(escflg),a	; Flag setzen
	; hlte1:	xor	a
	;		ld	(usrflg),a	; Ausgabe verhindern
	;		pop	af
	;		ret
	;
	; hlte2:	ld	a,(escflg)
	;		dec	a
	;		ld	(escflg),a	
	;		jr	nz,hlte1	; Erst nach 4. Eingabe 
	;		inc	a		; A=NZ
	;		ld	(usrflg),a	; freigeben
	;		pop	af
	;		ret
	;
	; In diesem Softwarepaket wird die Ausgabe der Statuszeile ueber
	; ein Flag (usrflg) gesteuert. Nur wenn das Flag NZ ist wird die
	; Statuszeile ausgegeben. Dies macht zwar die software etwas groesser,
	; verhindert aber unnoetige Ausgaben die nur das System verlangsamen
	; wuerden. So setzt der Uhren-Interrupt dieses Flag nur alle Minute
	; und die Statusausgabe von Diskette nur bei entsprechenden Aenderungen
	; der Ausgegebenen Werte
	;

	cseg

sndstl:	ld	a,(usrflg)	; Userflag gesetzt?
	or	a
	ret	z		; wenn nicht ...

	;
	; Ausgabe der Uhrzeit
	;
	
	ld	a,(@hour)	; lese Stunde
	call	bcd2asc
	ld	(sstd),hl
	ld	a,(@min)	; lese Minuten
	call	bcd2asc
	ld	(smin),hl

	;
	; Bedingungslose Ausgabe der Statuszeile
	;

sendit:	ld	hl,usrlin
	call	?pmsg		; Ausgeben
	xor	a
	ld	(usrflg),a	; Flag ruecksetzen
	ret


dsknam:	

	;
	; Ausgabe des Diskettennamens
	; Wird nach jedem MOVE aufgerufen
	; gefundener Diskettenname wird in Laufwerksspezifischen
	; Puffer uebertragen und in der Statuszeile ausgegeben
	;

	ld	a,(@fx##)
	cp	17		; search for first
	ret	nz		; nur dann ...   	

	push	hl
	push	de
	push	bc
	ld	a,(@drv##)	; gueltiges Laufwerk
	cp	'E'-'A'		; RAMDISK?
	jr	z,getna2
	cp	'F'-'A'		; Harddisk?
	jr	z,getna2
	ld	e,a
	ld	d,0		; als Offset
	ld	hl,namtab	; Vektor der Namenstabelle
	add	hl,de
	add	hl,de		
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	push	de		; rette Pufferadresse
	ld	hl,(@crdma##)	; Start DMA = evtl Start LABEL
	ld	bc,11		; Laenge des Namens (Label)
	ld	a,blank		; Kenner fuer Label
	cp	(hl)		; Label vorhanden?
	inc	hl		; Skip Label-Kenner
	jr	z,getna1	; ... wenn gefunden
	ld	h,d	
	ld	l,e		; sonst <HL>=Puffer
	inc	de		; und <DE>=Puffer+1
	ld	(hl),a		; 'alten' Namen loeschen
	dec	bc		; nur 10 Zeichen (1. ist schon in (hl))
getna1:	ldir			; Namen uebergeben
	pop	hl		; Pufferadresse
	ld	de,sname	; Zeiger in Statuszeile
	ld	bc,11		; Laenge
	ldir			; und Namen in Statuszeile ebenfalls ablegen
	ld	(usrflg),a	; erzwinge Ausgabe (A=NZ)
getna2:	pop	bc
	pop	de
	pop	hl
	ret

	dseg			; wird 'nur' aus Bank 0 aufgerufen

dsksta:

	;
	; wird nach jedem READ/WRITE aufgerufen
	; in (@DRV) steht benutztes Laufwerk und in (@FX) der anliegende
	; BDOS-Befehl
	;
	
	or	a		; Lesen/bzw Schreiben Fehlerfrei?
	ret	nz		; wenn nicht
	
	ld	a,(@drv##)	; benutztes Laufwerk
	ld 	hl,stldrv	; Zeiger auf 'letztes' Laufwerk
	cp	(hl)		; gleich?
	ld	(hl),a		; ... fuers naechste mal
	ld	e,a		; Laufwerksnummer als Offset nach <DE>
	ld	d,0
	jr	z,dsta1		; ... wenn angesprochenes Laufwerk='altes' LW
	
	;
	; wenn Laufwerk gewechselt wurde
	;

	push	de		; rette Offset
	ld	hl,namtab	; Zeiger auf Namenspuffer errechnen
	add	hl,de
	add	hl,de
	ld	a,(hl)		; ... nach <HL> Laden
	inc	hl
	ld	h,(hl)
	ld	l,a
	ld	de,sname	; Zeiger auf Labelfeld in Statuszeile
	ld	bc,11		; Laeng des Labels (8+3)
	ldir			; in Statuszeile schreiben
	pop	de		; Offset  (BC=0 nach LDIR!)	

dsta1:	ld	hl,@dtbl##	; Tabelle der XDPH-Zeiger
	add	hl,de
	add	hl,de		; Berechne Zeiger fuer gueltigen XDPH
	ld	a,(hl)		; (zu dem Laufwerk, dass gerade benutzt wurde)
	inc	hl
	ld	h,(hl)
	ld	l,a		; Zeiger auf XDPH in <HL>

	;
	; Berechnung des freien Speicherplatzes auf dem gueltigen Laufwerk
	; HL=XDPH-Vektor  E=Laufwerk
	;

	ld	a,e
	add	a,'A'
	ld	(sdisk),a	; Laufwerksnummer
	ld	(usrflg),a	; Ausgabe erzwingen (A=NZ)
	ld	de,12		; Offset zu DPB-Zeiger
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)		; Zeiger auf DPB nach <DE>
	inc	hl
	inc	hl
	inc	hl		; ALLOC-Vektor
	ld	a,(hl)
	inc	hl
	ld	h,(hl)
	ld	l,a		; Zeiger auf ALLOC-Adresse in <HL> 
	push	hl		; retten
	ex	de,hl		; und 'eigenen' dpb aufbauen
	ld	de,xxdph
	ld	bc,7		; aber nur bis 'dsm' wichtig
	ldir
	pop	de		; ALLOC-Vektor
	ld	hl,(xxdsm)	; volle Kapazitaet
	ld	b,3		; /8
gfr1:	or	a		; Reset Carry
	rr	h
	rr	l
	djnz	gfr1
	inc	hl		; Ausgleich
	ld	bc,0		; Blockzaehler ruecksetzen
gfr2:	ld	a,(de)		; Byte aus ALOC
gfr3:	or	a		; belegt?
	jr	z,gfr5

gfr4:	rra
	jr	nc,gfr4		; shift bis Bit gesetzt
	inc	bc		; dann Blockzaehler +1
	jr	gfr3		; bis Byte 'leer'

gfr5:	inc	de		; Naechstes Byte
	dec	hl		; Blockzaehler
	ld	a,l
	or	h		; letzter Block?
	jr	nz,gfr2		; wenn nicht ...
	
	ld 	hl,(xxdsm)	; xxdsm=volle Kapazitaet
	inc	hl
	or	a
	sbc	hl,bc		; - belegter Kapazitaet
	ld	a,(xxbsh)	; Faktor der Blockgroesse
	dec	a
	dec	a
gfr6:	dec	a
	jr	z,gfr7
	add	hl,hl		; sonst Blockgroesse*Kapazitaet		
	jr	gfr6

gfr7:	push	ix
	ld	ix,skap		; Zeiger auf Textbuffer
	xor	a
	ld	(first),a

	;
	; Umwandeln in Dezimal
	;

	ld	bc,table10##
	ld	de,-10000	
getf3:	ld	a,'0'-1
getf4:	push	hl
	inc	a
	add	hl,de
	jr	nc,getf5
	inc	sp
	inc	sp
	jr	getf4

getf5:	call	insert
getf6:	pop	hl
	ld	a,(bc)
	ld	e,a
	inc	bc
	ld	a,(bc)
	ld	d,a
	inc	bc
	ld	a,e
	or	d
	jr	nz,getf3
	pop	ix
dsdone:	xor	a		; Fehlermeldung = 00
	ret
	
insert:	ld	(ix+0),a	; Zeichen in Puffer
	cp	'0'		; war es eine Null?
	jr	nz,ins1		; ... wenn nicht
	ld	a,(first)	; sonst pruefen
	or	a		; ob fuehrende Null
	jr	nz,ins1		; ... wenn nicht
	ld	(ix+0),blank	; sonst Blank einfuegen
	jr	ins2

ins1:	ld	(first),a	; reset FIRST-Flag
ins2:	inc	ix
	ret

first:	db	0
stldrv:	db	-1
xxdph:	ds	2
xxbsh:	ds	3
xxdsm:	ds	2

   	cseg

namtab:	dw	nbuffa
	dw	nbuffb
	dw	nbuffc
	dw	nbuffd
	dw	nbuffe
	dw	nbufff
	dw	nbuffg
	dw	nbuffh
	dw	nbuffi
	dw	nbuffj
	dw	nbuffk
	dw	nbuffl
	dw	nbuffm
	dw	nbuffn
	dw	nbuffo
	dw	nbuffp

nbuffa:	ds	11,blank
nbuffb:	ds	11,blank
nbuffc:	ds	11,blank
nbuffd:	ds	11,blank
nbuffe:	db	'RAMDISK    '
nbufff:	db	'HARDDISK   '
nbuffg:	ds	11,blank
nbuffh:	ds	11,blank
nbuffi:	ds	11,blank
nbuffj:	ds	11,blank
nbuffk:	ds	11,blank
nbuffl:	ds	11,blank
nbuffm:	ds	11,blank
nbuffn:	ds	11,blank
nbuffo:	ds	11,blank
nbuffp:	ds	11,blank

	;
	; ****************
	; Statuszeile 
	; ****************
	;

usrlin:	db	esc,'f' 	; Einschalten Statuszeile	

usrbeg:	db	blank		; Start der Statuszeile		

sday:	db	'  .'		; Tag
smonth:	db	'  .'		; Monat
syear:	db	'00  '		; 
sstd:	db	'00:'		; Stunden
smin:	db	'00  '		; Minuten
	db	'Laufwerk: '
sdisk:	db	'A '
skap:	db	'     k frei   '
sname:	db	'              '
	dc	cr

