; ****************************************************************
; *								 *
; * MODUL		TIME.INC				 *
; * REV 1.0		850217					 *
; *								 *
; * Einbindung der verschiedenen Uhrenmodule in das CP/M 3 BIOS  *
; *								 *
; * Unterstuetzt wird die software-Uhr mit Print CTC oder die    *
; * hardware-Uhren auf der PRINT-UHR bzw der CPU85SC		 *
; *								 *
; * Alle Uhren laufen mit einem 1 sec Interrupt			 *
; *								 *
; * Nach jedem Interrupt wird die Zeit (im SCB) neu gestellt und *
; * das flag USRFLG auf NULL zurueckgesetzt.			 *
; *								 *
; * geschrieben von RAOUL O. KOERBER, Detmold			 *
; *								 *
; ****************************************************************

	;
	; ****************
	; Vereinbarungen
	; ****************
	;

ycount	equ	0b6ah		; Zaehler 1.1.86 (jedes Jahr +365!)

sc1	equ	00000001b	; software-Uhr auf Print CTC
hc1	equ	00000010b	; hardware-Uhr auf Print-UHR
hc2	equ	00000100b	; hardware-Uhr auf CPU85SC
	
p$ciot	equ	07h		; Portadresse des CIO-timers

	;
	; ****************
	; Welche Uhr ?
	; ****************

   IF	Z80ASM			; Abfrage erfolgt NUR wenn ISCLOCK=JA

	.printx
	.printx	Es koennen folgende Zeitgeber (Uhren) eingebaut werden
	.printx
	.printx      SC1   software-Uhr auf Print-CTC
	.printx      HC1   hardware-Uhr auf Print-UHR
        .printx	     HC2   hardware-Uhr auf CPU85SC
	.printx
	.accept 'Bitte entsprechende Angaben: ',CLOCK
	.printx
	.printx Es kann eine Statuszeile als 25. Zeile eingebunden werden
	.printx Hierzu ist VIDEO-80 oder ein entsprechendes Terminal notwendig
	.printx
	.accept	'Statuszeile einbinden     (JA-NEIN) ',STATUS
    
    ELSE

CLOCK	EQU	NEIN
STATUS	EQU	NEIN

   ENDIF

	;
	; ****************
	; Init der UHR
	; ****************
	;

	;
	; Aufruf von CLKINI beim Kaltstart
	;

clkini:

	;
	; Service-Zeiger initialisieren
	;

	di			; Interupt (noch) sperren
	ld	hl,tserv	; Zeiger

   if	clock 	ne hc1

	ld	(ivect1),hl	; ablegen		

   else	

        ld	(ivect0),hl	; ablegen

   endif


	ld	a,0ffh		; high Byte
	ld	i,a		; in's I-register
	im	2		; und Modus 2 einschalten

	;
	; nun Update der Zeit im SCB
	;

	;
	; erst mal Jahr aus Vorgabe berechenen
	;

	ld	hl,ycount	; Jahresanfang (Vorgabe)
	call	gets0		; Jahreszahl steht dann in tpby
	
   if	clock 	ne sc1		;  hardware-Uhr einlesen

	;
	; Uhrzeit einlesen
	;

   if	clock	eq hc1
	ld	hl,ipb		; Uhrentreiber initialisieren
	push	hl
	call	i2cini
	pop	hl
   endif

	ld	hl,tpbm		; nun Uhrzeit in tbp lesen
	push	hl
	call	clkget			
	pop	hl

  endif				; des Einlesens

	;
	; jetzt DATE berechnen
	;

	ld	a,(tpby)	; Jahr
	call	bcd2bin		; in binaer wandeln

clki1:	ld	b,a		; Jahr in B retten
	and	11111100b	; Schaltjahr ?
	cp	b
	ld	a,29		; dann 29 Tage
	jr	z,clki2
	dec	a		; sonst 28 Tage
clki2:	ld	(feb),a		; im Februar
	ld	hl,0		; dies ist der 1.1.78 (0)
clki3:	ld	a,b
	cp	78		; auf 1978 zurueckgezaehlt?
	jr	z,clki5		; dann (soweit) fertig
	ld	de,366		; fuer Schaltjahr
	and	00000011b	; ist es eines ?
	jr	z,clki4		; dann Sprung
	dec	de		; sonst 365 Tage verwenden
clki4:	add	hl,de		; Tage aufaddieren
	dec	b		; Jahr -1
	jr	clki3	

clki5:	ld	a,(tpbm)	; Monatszaehler
	call	bcd2bin		; in binaer
	ld	b,0		; fuer Addition
	ld	de,montab
clki6:	dec	a		; erster Monat?
	jr	z,clki7
	ex	de,hl
	ld	c,(hl)		; Monatstage lt Tabelle
	ex	de,hl
	add	hl,bc		; addieren
	inc	de		; Monatstabelle +1
	jr	clki6

clki7:	ld	a,(tpbd)	; nun noch den Tag
	call	bcd2bin
	ld	c,a
	ld	b,0
	add	hl,bc		; dazu addieren
	ld	(@date),hl	; und im SCB ablegen

	;
	; nun noch Stunden und Minuten ablegen
	;

	ld	hl,tpbh
	ld	de,@hour
	ld	b,3
clki8:	ld	a,(hl)
	ld	(de),a
	inc	hl
	inc	de
	djnz	clki8

	;
	; Sekunden-Interrupt initialisieren
	;

   if	clock	eq hc2		; wenn CPU85SC die CIO

	in	a,(p$ciot)
	xor	a
	out	(p$ciot),a
	in	a,(p$ciot)
	xor	a
	out	(p$ciot),a
	inc	a
	out	(p$ciot),a
	dec	a
	out	(p$ciot),a
	ld	hl,cioini
	ld	bc,ci$len shl 8 + p$ciot
	otir

	ld	a,00001011b	; trigger gate IE
	out	(p$ciot),a
	ld	a,11000110b
	out	(p$ciot),a
   
   else				; sonst CTC initialisieren
    if	clock	eq sc1

	ld	hl,ctcini
	call	send$init
    
    endif
   endif

   if	status

	call	ustat

   endif

	ei			; Interrupt freigeben
	ret

	;
	; ****************
	; Tabellen
	; ****************
	;

   if	clock	eq hc2		; fuer CIO

cioini: db	0		; MI
	db	10000100b	; CT VIS MIE

	db	00000100b	; CIV
	db	00h		; IVECT0 !!!!!!

	db	00010110b	; CT1 msb
	db	27h		; TC 

	db	000101110b	; CT1 lsb
	db	10h		; TC 

	db	00011100b	; CT1 Mode
	db	10000010b	; cont sq wave 0.01s (100H)

	db	00011000b	; CT2 msb
	db	0		; TC 

	db	00011001b	; CT2 lsb
	db	100		; 100*0.01s=1s

	db	00011101b	; CT2 MR
	db	11000000b	; cont mode, pulse

	db	00000001b	; MC
	db	01100011b	; CT1E,CT2E,CT1 count=CT2

	db	00001011b	; CC & S
	db	01100000b	; IE

	db	00001010b	; CT1 C & S
	db	00000110b	; no Int, Gate, Trigger

ci$len	equ	$-cioini

   else

ctcini:	db	7		; Inistringlaenge/2
	db	p$ctc0,resctc
	db	p$ctc1,resctc
	db	p$ctc0, low ivect0
	db	p$ctc0,timer
	db	p$ctc0,tim40
	db	p$ctc1,counter
	db	p$ctc1,cntcon

   endif

	;
	; ****************	
	; Service
	; ****************
	;

	cseg			; im gemeinsamen Speicherbereich

tserv:

   if	clock	ne hc2		; wenn CTC

	di
	push	af
	push	hl		; verwende Register retten
	ld	hl,@sec		; Sekundenpuffer im SCB
	ld	a,(hl)		; Zeit lesen
	add	a,1		; +1
	daa			; in dezimal rueckwandeln
	ld	(hl),a		; zuruecklegen
	cp	60h		; Minute erreicht
	jr	c,exit1		; ... wenn noch nicht

	ld	(hl),0		; sonst Sekundenzeiger ruecksetzen
	dec	hl		; und Zeiger auf @MIN
	ld	a,(hl)
	add	a,1
	daa
	ld	(hl),a
	cp	60h
	jr	c,exit		; sie oben ...

	ld	(hl),0		; sonst Minutenzeiger ruecksetzen
	dec	hl		; und Zeiger auf Stunden
	ld	a,(hl)
	add	a,1
	daa
	ld	(hl),a
	cp	24h		; Mitternacht?
	jr	c,exit

	ld	(hl),0		; sonst Stundenzeiger auf 0-Uhr
	ld	hl,(@date)	; sonst Datum
	inc	hl		; +1 Tag
	ld	(@date),hl	; neues Datum

exit:	ld	(usrflg),a	; USRFLG freigeben (A=NZ)
exit1:	pop	hl
	pop	af
	ei
	reti
 
   else				; falls HC2
	
	di
	push	af
	push	hl		; verwendete Register retten
	
	;
	; Rest Gate
	;

	ld	a,00001011b
	out	(p$ciot),a
	ld	a,00100000b
	out	(P$ciot),a

	;
	; Update Zeit
	;

	ld	hl,@sec		; Sekundenpuffer im SCB
	ld	a,(hl)		; Zeit lesen
	add	a,1		; +1
	daa			; in dezimal rueckwandeln
	ld	(hl),a		; zuruecklegen
	cp	60h		; Minute erreicht
	jr	c,exit1		; ... wenn noch nicht

	ld	(hl),0		; sonst Sekundenzeiger ruecksetzen
	dec	hl		; und Zeiger auf @MIN
	ld	a,(hl)
	add	a,1
	daa
	ld	(hl),a
	cp	60h
	jr	c,exit		; sie oben ...

	ld	(hl),0		; sonst Minutenzeiger ruecksetzen
	dec	hl		; und Zeiger auf Stunden
	ld	a,(hl)
	add	a,1
	daa
	ld	(hl),a
	cp	24h		; Mitternacht?
	jr	c,exit

	ld	(hl),0		; sonst Stundenzeiger auf 0-Uhr
	ld	hl,(@date)	; sonst Datum
	inc	hl		; +1 Tag
	ld	(@date),hl	; neues Datum
exit:	ld	(usrflg),a	; USRFLG freigeben (A=NZ)
exit1:	ld	a,00001011b	; CIO Interrupt wieder freigeben	
	out	(p$ciot),a
	ld	a,00100100b
	out	(p$ciot),a
	pop	hl
	pop	af
	ei
	ret			; CIO kennt kein RETI !!!	
   
   endif
	

	dseg

	;
	; ****************
	; UHR setzen und lesen
	; ****************
	;

?time: 	inc	c		; Uhrzeit in SCB schreiben?
	ret	nz		; ... das wird vom Interrupt uebernommen

	;
	; Uhrzeit stellen entsprechend den Daten im SCB
	; 

	di
	push	hl
	push	de
	call	getscb		; Daten aus SCB in den tpb schreiben

   if	clock	ne sc1		; nur wenn hardware-Uhr vorhanden

	ld	hl,tpbm		; nun TPB-Adresse
	push	hl		; auf den Stack
	call	clkset		; und Uhr setzen
	pop	hl

   endif

	pop	de
	pop	hl
	ei
	ret

	;
	; Uhrzeit aus SCB in tpb schreiben und Datum in Statuszeile
	;

getscb:	ld	hl,(@date)	; Datum
gets0:	ld	b,78		; Start-Jahr
gets1:	ld	a,b
	and	00000011b	; Schaltjahr?
	ld	de,366		; dann 366 Tage
	jr	z,gets2
	dec	de		;  sonst 365 Tage
gets2:	ld	(rest),hl	
	sbc	hl,de		; 
	jr	c,gets3		;  wenn Jahr gefunden
	inc	b		; Jahr +1
	ld	a,h
	or	l		;  genau am Jahresende ?
	jr	nz,gets1
	ld	(rest),hl

	;
	; Jahr ist in <B>
	;

gets3:	ld	a,b		; 
	and	00000011b	; Ist Jahr ein Schaltjahr ?
	ld	a,29		; dann 29 Tage im Februar
	jr	z,gets4
	dec	a		; sonst 28 Tage
gets4:	ld	(feb),a		; in Tabelle ablegen
	ld	a,b		; Jahr
	call	bin2bcd		; in BCD wandeln
	ld	(tpby),a	; und ebenfalls ablegen

	;
	; nun Monat berechnen
	;

	
	ld	hl,(rest)	; Resttage
	ld	de,montab	; Monatstabelle
	xor	a		; A=0 -> Monatszaehler
	ld	b,a		; B=0 fuer Offset
gets5:	inc	a		; Monat +1
	ex	de,hl
	ld	c,(hl)		; BC=Monatstage
	ex	de,hl
	ld	(rest),hl	; letzten Wert ablegen
	or	a		; Reset Cary
	sbc	hl,bc		; Jahresrest-Monatstage
	inc	de		; Zeiger auf naechsten Monat
	jr	nc,gets5	
	call	bin2bcd		; in Binaer wandeln
	ld	(tpbm),a	; und ablegen
	
	;
	; nun Tag berechnen
	;

	ld	a,(rest)	; der Rest sind die verbleibenden Tage
	call	bin2bcd		; wandeln in binaer
	ld	(tpbd),a	; und ablegen

	;
	; Stunden
	;

	ld	a,(@hour)
	ld	(tpbh),a

	;
	; Minuten
	;

	ld	a,(@min)
	ld	(tpbmi),a

	;
	; Sekunden
	;

	ld	a,(@sec)
	ld	(tpbs),a
	inc 	a		; A=NZ
	ld	(usrflg),a	; und Ausgabe erzwingen

	;
	; update Statuszeile mit Datum
	;

 ustat:

   if	status

	ld	a,(tpby)	; Jahr
	call	bcd2asc
	ld	(syear),hl
	ld	a,(tpbm)	; Monat
	call	bcd2asc
	ld	(smonth),hl
	ld	a,(tpbd)	; Tag
	call	bcd2asc
	ld	(sday),hl

   endif

	ret

	;
	; Wandlung binaer in BCD
	;

bin2bcd:ld	b,-1
b2d:	inc	b
	sub	10
	jr	nc,b2d
	add	a,10
	ld	c,a
	ld	a,b
	rlca
	rlca
	rlca
	rlca
	or	c
	ret
	
	;
	; Wandlung BCD in binaer
	;

bcd2bin:ld	b,a
	and	11110000b	; oberes nibble
	rrca			; /2
	ld	c,a		; diesen Wert retten
	rrca			; /4
	rrca			; /8
	add	a,c		; /10
	ld	c,a		; Wert retten
	ld	a,b		; Originalwert
	and	00001111b	; davon unteres nibble
	add	a,c
	ret

   if	status

	cseg

	;
	; Wandlung BCD in ASCII Rueckgabe des ASCII-Wertes in HL
	;

bcd2asc:push	af		; Retten
	rrca
	rrca
	rrca
	rrca			; oberes nibble (Jahrzehnt)
	call	b2a
	ld	l,a
	pop	af
	call	b2a
	ld	h,a
	ret

b2a:	and	00001111b
	or	'0'
	ret	

   	dseg

   endif

rest:	ds	2

tpby:	db	86h		; Jahr
tpbm:	db	0		; Monat
tpbd:	db	0		; Tag
tpbh:	db	0		; Stunde
tpbmi:	db	0		; Minute
tpbs:	db	0		; Sekunde

montab:	db	31		; JAN
feb:	db	28,31,30,31,30
	db	31,31,30,31,30,31

   if	clock eq hc1

ipb:	db	0		; Vektor low
	db	128		; /128
	db	10111100b

   	INCLUDE	CLOCK1.INC

   endif
   if	clock eq hc2

	INCLUDE	CLOCK2.INC

   endif
