	title 	'CP/M 3 BIOS fuer ELZET-80 Computer'
	subttl 	V83  Treiber fuer VIDEO-80
	page
; ****************************************************************
; *								 *
; * MODUL	V83.INC						 *
; * REL		2.1		851202				 *
; *								 *
; * Aufgabe dieses Treibers ist es mit einer ELZET-80 VIDEO-80-	 *
; * Karte eine weitestgehende Simulation eines TVI 912/920-	 *
; * Terminals im gebankten Bereich zu ermoeglichen		 *
; * Dieser Treiber ist als Erweiterung des nur auf das Not-	 *
; * wendigste beschraenkten V80.Z80 (MAC) Treibers gedacht.	 *
; * 								 *
; * Bedingt durch das was dieser Treiber 'kann' ist er leider	 *
; * etwas langsamer und wesentlich groesser. Da die Groesse im	 *
; * gebankten Bereich 'untergeht',ist dies jedoch unwesentlich	 *
; *								 *
; * geschrieben von RAOUL O. KOERBER, Detmold			 *
; *								 *
; ****************************************************************

	;
	; ****************
	;  Macros
	; ****************
	;

storec	macro			;; ablegen eines Zeichens
	out	(p$vtog),a	
	ld	(hl),a		
	out	(p$vtog),a	
	endm	

movec	macro			;; ablegen eines Zeichenblocks
	out	(p$vtog),a
	ld	(hl),a	
	ldir		
	out	(p$vtog),a
	endm

moves	macro			;; ablegen einer Zeichenkette
	out	(p$vtog),a
	ldir		
	out	(p$vtog),a	
	endm

moveu	macro			;; Ablegen Zeichenkette revers
	out	(p$vtog),a
	lddr		
	out	(p$vtog),a	
	endm	

	cseg			; gemeinsamer Bereich

	;
	; ****************
	; * Treiber	 *
	; ****************
	;

v80out:

	;
	; V80OUT erwartet das auszugebende Zeichen in <C> 
	; Auf eine Rueckgabe in <A> wird verzichtet, da dies
	; in der 'gemeinsamen' CONOUT-Routine uebernommen wird
	; Alle Register werden gerettet und ein 'eigener' Stack
	; deklariert
	;

	di			; keine Unterbrechnung
	ld	(savsp),sp	; rette Benutzerstack
	ld	sp,vstack	; und benutze eigenen
	ld	a,(@cbnk##)	; momentane Bank
	ld	(curbnk),a	; retten
	xor	a		; Bank 0
	call	?bank##		; schalten
	call	vout		; Videotreiber aufrufen
	db	3EH		; LD A, ... 
curbnk:	db	0		; Originalbank
	call	?bank##		; zurueckschalten
	db	31H		; LD SP, ....
savsp:	dw	0		; wird Benutzer-SP
	ei
	ret

	ds	32		; 16 calls oder push tief
vstack	equ	$

	; 
	; der 'eigentliche' Treiber verschwindet in der Bank
	;

	dseg			; im gebankten Bereich
	
vout:	push	hl
	push	de
	push	bc		
	ld	hl,vret		; Return aus Unterprogramm
	push	hl		; auf den Stack
	ld	a,(nxtflg)	; Test ob in Abfragesequenz
	or	a
	ld	a,c		; Zeichenuebergabe in <A>
	jr	z,vout1		; wenn nicht in NEXT-Sequenz, sonst
	db	0c3h		; JUMP zum
njump:	dw	0		; anforderdernden Unterprogramm
	
	;
	; **** Bildschirmbearbeitung
	;

vout1:	cp	blank		; Test ob CNTRL-Zeichen
	jr	c,control	; Sprung wenn JA
	
	;
	; **** Zeichenausgabe
	;

	ld	a,(invflg)	; Zeichen INVERS?
	or	c		; ... dann INVERS-Bit einfuegen
	ld	hl,(curpos)	; Cursorposition feststellen
	storec			; und Zeichen ablegen
	ex	de,hl		; CUR$POS in <DE>
	ld	hl,-800H	; Offset in Attributspeicher
	add	hl,de		; berechnen
	ld	a,(abits)	; Attribut einlesen
	storec			; und in Speicher uebernehmen
	ex	de,hl
	jp	cur$rechts	; Cursor setzen
	

	;
	; **** ESC-Verteiler
	;

escape:

	;
	; Hier wird zuerst das zweite (naechste) Zeichen zur Erkennung
	; einer ESC-Sequenz angefordert. Dieses Zeichen wird mit einer
	; Tabelle verglichen. Wird ein entsprechender Eintrag gefunden
	; wird das Unterprogramm angesprungen andernfalls wird das 
	; Zeichen OHNE AKTION zurueckgegeben
	;

	call	next		; hole naechstes Zeichen
	cp	7fh		; mehr Zeichen sind nicht zugelassen
	ret	nc

	;
	; **** CNTRL-Verteiler
	;

control:

	;
	; Hier wird die Eingabe der Kontrollzeichen gegen eine
	; Tabelle verglichen, wird ein entsprechender Eintrag gefunden
 	; andernfalls wird das Zeichen OHNE AKTION zurueckgegeben
	;

	ld	hl,ctrl$table	; In Vergleichstabelle 
	sub	bell		; unterste Grenze
	ret	c		; ... unerlaubtes Zeichen
	ld	e,a		; Zeichenuebergabe 
	ld	d,0		; nach <DE>
	add	hl,de
	add	hl,de		; 2* da Adresse gesucht !
	ld	a,(hl)		; Adresse nach <HL>
	inc	hl
	ld	h,(hl)
	ld	l,a
	push	hl		; auf den Stack
	ld	hl,abits	; wird oefters gebraucht
dummy:	ret

	;
	; **** Hilfsprogramme
	;

verpos:

	;
	; Berechnet die Speicherposition jedes Zeilenanfanges
	; aus VRAM und (CURVER)
	;

	ld	a,(curver)	; aktuelle V-Position einlesen
verpos1:ld	hl,vram		; RAM-Start
	ld	bc,ncol		; Spalten'laenge'
verpos2:dec	a		; countdown
	ret	m		; ... fertig
	add	hl,bc		; Position berechnen
	jr	verpos2		; bis fertig

is$bell:

	;
	; z.Z. nicht implementiert
	;

	ret

	;
	; **** Einsprungtabelle
	;

ctrl$table:


   if	z80asm

t920	defl	1
t925	defl	2
reserve defl	3

	.printx
	.printx
	.printx	Es koennen mehrere Terminal-Emulationen eingebunden werden:
	.printx 
	.printx     T920     Televideo 912/920
	.printx     T925     Televideo 925/950
        .printx     
	.accept	'bitte vorangestellte Zeichenkombination eingeben ',TERMINAL

   else

TERMINAL EQU	T920

   endif


	;
	; Steuerzeichen die 'belegt' sind
	; mit ** gezeichnete Funktionen sind zusaetzlich
	; mit dw DUMMY gezeichnete Funktionen sind unbelegt
	; oder nicht installiert
	;

   if	terminal eq t920

	dw	is$bell		; 07 	BELL
	dw	cur$links	; 08 	Backspace
	dw	dummy		; 09 	TAB
	dw	linefeed	; 0A 	lf
	dw	cur$up		; 0B 	cursur hoch
	dw	cur$rechts	; 0C 	cursor rechts
	dw	cur$cr		; 0D 	cr
	dw	dummy		; 0E 	-
	dw	dummy		; 0F 	-
	dw	dummy		; 10 	-
	dw	dummy		; 11 	-
	dw	dummy		; 12 	-
	dw	dummy		; 13 	-
	dw	dummy		; 14 	-
	dw	dummy		; 15 	-
	dw	dummy		; 16 	- 
	dw	dummy		; 17 	-
	dw	dummy		; 18 	-
	dw	dummy		; 19 	-
	dw	clear$all	; 1A 	clear screen
	dw	escape		; 1B 	Escape Sequenz lead-in
	dw	dummy		; 1C 	-
	dw	dummy		; 1D 	-
	dw	cur$home	; 1E 	Home cursor
	dw	new$line	; 1F 	Newline (CR-LF)

esc$table:

	;
	; Einsprungtabelle fuer ESC-Sequenzen
	;

	dw	dummy		; blank	-
	dw	dummy		; !
	dw	dummy		; "     unlock keyboard
	dw	dummy		; #     lock keyboard
	dw	dummy		; $	
	dw	dummy		; %     -
	dw	dummy		; &	protekt mode on
	dw	dummy		; '	protekt mode off
	dw	dim$off		; )	halbe Helligkeit off
	dw	dim$on		; (	halbe Helligkeit on
	dw	clear$all	; *	clear all to NULLs
	dw	clear$all	; +	clear all to SPACE
	dw	dummy		; ,	
	dw	dummy		; -	Adresse Cursor RAGE,ROW, COL
	dw	cur$attr	; . **	set cursor Attributs (TVI925/50)
	dw	dummy		; /	Read Cursor PAGE,ROW,COL
	dw	dummy		; 0	-
	dw	dummy		; 1	set col tab
	dw	dummy		; 2	clear col tab
	dw	dummy		; 3	clear all tabs
	dw	dummy		; 4	send line unprotected
	dw	dummy		; 5	send page unprotected
	dw	dummy		; 6	send line all
	dw	dummy		; 7	send page all
	dw	dummy		; 8
	dw	dummy		; 9
	dw	clear$all	; :	clear unprot to NULL
	dw	clear$all	; ;	clear unprot to SPACE
	dw	dummy		; <
	dw	cur$addr	; =	Cursor-Adressierung
	dw	dummy		; >
	dw	dummy		; ?	read cursor address ROW,COL
	dw	dummy		; @	extension mode on
	dw	dummy		; A	extension mode off
	dw	dummy		; B	Block mode ON
	dw	dummy		; C	conversation mode ON
	dw	dummy		; D	
	dw	insert$line	; E	line insert
	dw	dummy		; F
	dw	dummy		; G	
	dw	dummy		; H
	dw	dummy		; I	back tab
	dw	dummy		; J	
	dw	dummy		; K	Advance page
	dw	dummy		; L
	dw	dummy		; M
	dw	dummy		; N
	dw	dummy		; O
	dw	dummy		; P	print page
	dw	char$ins	; Q	character insert
	dw	delete$line	; R	line delete
	dw	dummy		; S	send message unprotected only
	dw	clear$eol	; T	erase to end of line (Blank)
	dw	dummy		; U
	dw	dummy		; V	-
	dw	char$del	; W	delete character
	dw	dummy		; X
	dw	clear$eos	; Y	erase to end of Page
	dw	zs$sel		; Z **  Zeichensatz umdefinieren
	dw	dummy		; [	-
	dw	dummy		; \	-
	dw	dummy		; ]	-
	dw	blk$on		; ^	start Blink field
	dw	dummy		; _	start Blank field
	dw	dummy		; `
	dw	dummy		; a
	dw	dummy		; b
	dw	dummy		; c
	dw	dummy		; d
	dw	dummy		; e
	dw	sline$set	; f ** 	setzen USER-Zeile bis CR
	dw	sline$on	; g **	anstellen USER-Zeile 
	dw	sline$off	; h **	abstellen USER-Zeile
	dw	dummy		; i 	Field tabs
	dw	inv$on		; j 	reverse Video
	dw	inv$off		; k 	End of reverse Video
	dw	dummy		; l 	Start underline
	dw	dummy		; m 	End underline
	dw	dummy		; n
	dw	dummy		; o
	dw	dummy		; p
	dw	blk$off		; q 	End BLINK/Blank field
	dw	dummy		; r
	dw	dummy		; s 	send message all
	dw	clear$eol	; t 	loeschen bis Zeilenende (NULL)
	dw	dummy		; u
	dw	dummy		; v 	AUTO page ON
	dw	dummy		; w 	AUTO page off
	dw	dummy		; x
	dw	clear$eos	; y 	loeschen bis Bildschirmende
	dw	zs$sel		; z ** 	Zeichensatz umdefinieren
	dw	dummy		; {
	dw	dummy		; |
	dw	dummy		; }
	dw	dummy		; ~
	dw	dummy		; DEL

   endif

   if	terminal eq t925

	dw	is$bell		; 07 	BELL
	dw	cur$links	; 08 	Backspace
	dw	dummy		; 09 	TAB
	dw	linefeed	; 0A 	lf
	dw	cur$up		; 0B	cursur hoch	
	dw	cur$rechts	; 0C 	cursor rechts
	dw	cur$cr		; 0D 	cr
	dw	dummy		; 0E 	XON/XOFF Disable
	dw	dummy		; 0F 	XON/XOFF Enable
	dw	dummy		; 10 	-
	dw	dummy		; 11 	-
	dw	dummy		; 12 	-
	dw	dummy		; 13 	-
	dw	dummy		; 14 	-
	dw	dummy		; 15 	-
	dw	cur$down	; 16 	- cursor runter
	dw	dummy		; 17 	-
	dw	dummy		; 18 	-
	dw	dummy		; 19 	-
	dw	clear$all	; 1A 	clear screen
	dw	escape		; 1B 	Escape Sequenz lead-in
	dw	dummy		; 1C 	-
	dw	dummy		; 1D 	-
	dw	cur$home	; 1E 	Home cursor
	dw	new$line	; 1F  	Newline

esc$table:

	;
	; Einsprungtabelle fuer ESC-Sequenzen
	;

	dw	dummy		; blank	- lead-in LOAD/READ time
	dw	dummy		; !
	dw	dummy		; "     unlock keyboard
	dw	dummy		; #     lock keyboard
	dw	dummy		; $	
	dw	dummy		; %     -
	dw	dummy		; &	protekt mode on
	dw	dummy		; '	protekt mode off
	dw	dim$off		; )	halbe Helligkeit off
	dw	dim$on		; (	halbe Helligkeit on
	dw	clear$all	; *	clear all to NULLs
	dw	clear$all	; +	clear all to SPACE
	dw	dummy		; ,	clear all to half intensity
	dw	dummy		; -	Cursor set PAGE,ROW,COL	
	dw	cur$attr	; .	set cursor Attributs
	dw	dummy		; /	Cursor pos read PAGE,ROW,COL-
	dw	dummy		; 0	-
	dw	dummy		; 1	set col tab
	dw	dummy		; 2	clear col tab
	dw	dummy		; 3	clear all tabs
	dw	dummy		; 4	send line unprotected
	dw	dummy		; 5	send page unprotected
	dw	dummy		; 6	send line all
	dw	dummy		; 7	send page all
	dw	dummy		; 8
	dw	dummy		; 9
	dw	clear$all	; :	clear unprot to NULL
	dw	clear$all	; ;	clear unprot to SPACE
	dw	dummy		; <	Keyklick on
	dw	cur$addr	; =	Cursor-Adressierung
	dw	dummy		; >	Keyklick off
	dw	dummy		; ?	read cursor address
	dw	dummy		; @	extension mode on
	dw	dummy		; A	extension mode off
	dw	dummy		; B	Block mode ON
	dw	dummy		; C	conversation mode ON
	dw	dummy		; D	
	dw	insert$line	; E	line insert
	dw	dummy		; F
	dw	set$attr	; G	Attribut leadin
	dw	dummy		; H
	dw	dummy		; I	back tab
	dw	inv$on		; J	Back page
	dw	inv$off		; K	advance page
	dw	dummy		; L	
	dw	dummy		; M	
	dw	dummy		; N	normal screen
	dw	dummy		; O	blank screen
	dw	dummy		; P	print page
	dw	char$ins	; Q	character insert
	dw	delete$line	; R	line delete
	dw	dummy		; S	send message
	dw	clear$eol	; T	erase to end of line
	dw	dummy		; U	Monitor mode on
	dw	dummy		; V	-
	dw	char$del	; W	delete character
	dw	dummy		; X	Monitor mode off
	dw	clear$eos	; Y	erase to end of Page
	dw	zs$sel		; Z **  USER-ROM (Zeichensaetze)
	dw	dummy		; [	-
	dw	dummy		; \	-
	dw	dummy		; ]	-
	dw	dummy		; ^	-
	dw	dummy		; _	transparent print on
	dw	dummy		; `	-
	dw	dummy		; a     transparent print off
	dw	dummy		; b	Black on white
	dw	dummy		; c
	dw	dummy		; d	White on Black
	dw	dummy		; e
	dw	sline$set	; f 	setzen USER-Zeile bis CR
	dw	sline$on	; g 	USER-Zeile EIN 
	dw	sline$off	; h 	USER-Zeile AUS
	dw	dummy		; i	field tabs
	dw	rev$lf		; j 	reverse linefeed
	dw	dummy		; k	set local edit mode
	dw	dummy		; l	set duplex edit mode
	dw	dummy		; m
	dw	dummy		; n
	dw	dummy		; o
	dw	dummy		; p	set print termination char
	dw	blk$off		; q
	dw	dummy		; r
	dw	dummy		; s	send message all
	dw	clear$eol	; t 	loeschen bis Zeilenende
	dw	dummy		; u	Monitor mode OFF
	dw	dummy		; v	Auto page on
	dw	dummy		; w	Auto page off
	dw	dummy		; x
	dw	clear$eos	; y 	loeschen bis Bildschirmende
	dw	zs$sel		; z 	USER-ROM (Zeichensaetze)
	dw	dummy		; {
	dw	dummy		; |
	dw	dummy		; }
	dw	dummy		; ~
	dw	dummy		; DEL

   endif
   if	terminal eq reserve

	dw	is$bell		; 07 	BELL
	dw	cur$links	; 08 	Backspace
	dw	dummy		; 09 	-
	dw	linefeed	; 0A 	lf
	dw	cur$up		; 0B 	cursur hoch
	dw	cur$rechts	; 0C 	cursor rechts
	dw	cur$cr		; 0D 	cr
	dw	dummy		; 0E 	-
	dw	dummy		; 0F 	-
	dw	dummy		; 10 	-
	dw	dummy		; 11 	-
	dw	dummy		; 12 	-
	dw	dummy		; 13 	-
	dw	dummy		; 14 	-
	dw	dummy		; 15 	-
	dw	linefeed	; 16	- cursor runter
	dw	dummy		; 17 	-
	dw	dummy		; 18 	-
	dw	dummy		; 19 	-
	dw	clear$all	; 1A 	clear screen
	dw	escape		; 1B 	Escape Sequenz leadin
	dw	dummy		; 1C 	-
	dw	dummy		; 1D 	-
	dw	cur$home	; 1E 	Home cursor
	dw	new$line	; 1F	Newline

esc$table:

	;
	; Einsprungtabelle fuer ESC-Sequenzen
	;

	dw	dummy		; blank	-
	dw	dummy		; !
	dw	dummy		; "     unlock keyboard
	dw	dummy		; #     lock keyboard
	dw	dummy		; $	
	dw	dummy		; %     -
	dw	dummy		; &	protekt mode on
	dw	dummy		; '	protekt mode off
	dw	dim$off		; )	halbe Helligkeit off
	dw	dim$on		; (	halbe Helligkeit on
	dw	clear$all	; *	clear all to NULLs
	dw	clear$all	; +	clear all to SPACE
	dw	dummy		; ,	clear all to half intensity
	dw	dummy		; -	-
	dw	cur$attr	; .	set cursor Attributs
	dw	dummy		; /	-
	dw	dummy		; 0	-
	dw	dummy		; 1	set col tab
	dw	dummy		; 2	clear col tab
	dw	dummy		; 3	clear all tabs
	dw	dummy		; 4	send line unprotected
	dw	dummy		; 5	send page unprotected
	dw	dummy		; 6	send line all
	dw	dummy		; 7	send page all
	dw	dummy		; 8
	dw	dummy		; 9
	dw	dummy		; :	clear unprot to NULL
	dw	clear$all	; ;	clear unprot to SPACE
	dw	dummy		; <	Keyklick on
	dw	cur$addr	; =	Cursor-Adressierung
	dw	dummy		; >	Keyklick off
	dw	dummy		; ?	read cursor address
	dw	dummy		; @	extension mode on
	dw	dummy		; A	extension mode off
	dw	dummy		; B	Block mode
	dw	dummy		; C	conversation mode
	dw	dummy		; D	
	dw	insert$line	; E	line insert
	dw	dummy		; F
	dw	set$attr	; G	Attribut leadin
	dw	dummy		; H
	dw	dummy		; I	back tab
	dw	inv$on		; J	reverse on (TVI 920)
	dw	inv$off		; K	reverse off (TVI 920)
	dw	dummy		; L	start underline
	dw	dummy		; M	stop underline
	dw	dummy		; N	normal screen
	dw	dummy		; O	blank screen
	dw	dummy		; P	print page
	dw	char$ins	; Q	character insert
	dw	delete$line	; R	line delete
	dw	dummy		; S	send message
	dw	clear$eol	; T	erase to end of line
	dw	dummy		; U	Monitor mode on
	dw	dummy		; V	-
	dw	char$del	; W	delete character
	dw	dummy		; X	Monitor mode off
	dw	clear$eos	; Y	erase to end of Page
	dw	dummy		; Z	-
	dw	dummy		; [	-
	dw	dummy		; \	-
	dw	dummy		; ]	-
	dw	blk$on		; ^	start Blink field
	dw	dummy		; _	start Blank field
	dw	dummy		; `
	dw	dummy		; a
	dw	dummy		; b
	dw	dummy		; c
	dw	dummy		; d
	dw	dummy		; e
	dw	sline$set	; f setzen USER-Zeile bis CR
	dw	sline$on	; g anstellen USER-Zeile 
	dw	sline$off	; h abstellen USER-Zeile
	dw	dummy		; i
	dw	rev$lf		; j reverse linefeed
	dw	dummy		; k
	dw	dummy		; l
	dw	dummy		; m
	dw	dummy		; n
	dw	dummy		; o
	dw	dummy		; p
	dw	blk$off		; q
	dw	dummy		; r
	dw	dummy		; s
	dw	clear$eol	; t loeschen bis Zeilenende
	dw	dummy		; u
	dw	dummy		; v
	dw	dummy		; w
	dw	dummy		; x
	dw	clear$eos	; y loeschen bis Bildschirmende
	dw	zs$sel		; z Zeichensatz umdefinieren
	dw	dummy		; {
	dw	dummy		; |
	dw	dummy		; }
	dw	dummy		; ~
	dw	dummy		; DEL

   endif 

	;
	; ****************
	; Unterprogramme
	; ****************
	;

clear$all:
	
	;
	; loeschen des Bildschirmes und des Attributspreichers.
	; Ist das A-Flag fuer den deutschen Zeichensatz gesetzt
	; so wird der Attibutspeicher mit dem entsprechenden Bit
	; 'geloescht' D.H. Bei CLS bleibt der Deutsche Zeichensatz
	; erhalten. Er kann nur durch explizites Umschalten auf
	; US-ASCII oder einen anderen Zeichensatz 'abgeschaltet'
	; werden.
	;

	call	aclear		; und loesche
	call	cur$home	; Cursor HOME
	ld	de,vram		; Start VRAM
	jr	eos0
	
clear$eos:
	
	;
	; Loeschen ab Cursorposition bis Bildschirmende
	;

	ld	de,(curpos)	; einlesen Cursorposition
eos0:	ld	hl,screen$end	; Zeiger auf Bildschirmende
eos1:	xor	a		; Carryflag loeschen <A>=0
	dec	hl
	sbc	hl,de		; Laengenberechnung
	ret	z		; CURPOS=ENDE
	ld	b,h		; und nach <BC> kopieren
	ld	c,l
	ld	h,d		; Cursorposition nach
	ld	l,e		; <HL> kopieren
	inc	de		; naechste Position in VRAM
	jr	acl1

aclear:

	;
	; loeschen des Attributspeichers 
	; der deutsche Zeichensatz bleibt erhalten
	;

	ld	hl,aram		; Zeiger auf Attributspeicher
	ld	d,h		; Kopie nach <DE>
	ld	e,l
	inc	de		; Naechste Position
	ld	bc,screen$len	; Laenge des Speichers
	ld	a,(abits)	; Attribute
	and	000010000b	; nur deutschen Zeichensatz zulassen
	ld	(abits),a
acl1:	movec			; und loeschen mit Z80 power
	ret
	
clear$eol:
	
	;
	; Loeschen bis Zeilenende
	;

	ld	a,(curver)	; V-pos des Cursors
	inc	a		; +1
	call	verpos1		; Berechne Beginn naechste Zeile
	ld	de,(curpos)	; Cursorposition
	jr	eos1

cur$cr:

	;
	; Cursor auf Zeilenanfang setzen
	;

	xor	a		; H-Pos = 0
	ld	(curhor),a
	ret
	
cur$home:		
	
	;
	; Cursor nach oben links ohne Bildschirm loeschen
	; oder Attributflags veraendern
	;

	ld	hl,null		; H-Pos UND V-Pos = 0
cur$h1:	ld	(curhor),hl	
	ret
	
cur$rechts:
	
	;
	; Cursor eine Positions nach rechts. Ist Position am
	; Zeilenende, wird er auf die erste Position der naechsten
	; Zeile gesetzt. Es wird gescrollt wenn der Cursor in der 
	; letzten Spalte der letzten Zeile ist 
	;

	ld	hl,curhor	; Zeiger auf H-Pos setzen
	inc	(hl)		; +1
	ld	a,(hl)		; Einlesen der Position
cur$r1:	cp	ncol		; am Ende der Zeile ?
	ret	c		; ... wenn nicht
cur$r2:	ld	(hl),0		; sonst Spalte 0
	jr	linefeed
	
cur$down:
	
	;
	; Cursor ein Zeile nach unten jedoch in gleiche Spalte
	;

	ld	hl,curver	; Zeiger auf V-Pos setzen
	inc	(hl)		; +1
	ld	a,(hl)		; Einlesen der Position
	cp	nrow		; Ende des Bildschirmes ?
	ret	c		; ... wenn nicht
	dec	(hl)		; sonst 'alte' Position und scrollen
	or	a		; reset Carry
	ret

new$line:

	;
	; CR-LF Sequenz
	;

	call	cur$cr

linefeed:

	;
	; wie CUR$DOWN jedoch SCROLL wenn in letzter Zeile
	;

	call	cur$down
	ret	c		; noch nicht im Ende	

scroll$up:
	
	;
	; Scrollen ( aufrollen ) des Bildschirmes
	; die oberste Zeile geht verloren unten wird eine
	; Leerzeile neu eingeschoben
	;
	;
	; zuerst Attribute bis auf unterste Zeile ...
	;

	ld	hl,aram+ncol	; Zeiger auf Start Zeile 2 ARAM
	ld	de,aram		; Zeiger auf Zeile 1 im ARAM
	ld	bc,screen$len-ncol ; Laenge ( ab Zeile 2 ... Ende )
	moves
	ld	hl,vram+ncol	; Zeiger auf Start Zeile 2
	ld	de,vram		; Zeiger auf Zeile 1
	ld	bc,screen$len-ncol ; Laenge ( ab Zeile 2 ... Ende )
	moves			; Transfer
	jp	eos0
	
cur$links:
	
	;
	; Cursor eine Position nach links, ist der Cursor in der 1. Spalte
	; springt er von dort in die letzte Spalte der darueberliegenden
	; Zeile. In der HOME-Position bleibt der Cursor stehen
	;

	ld	hl,curhor	; H-Pos
	dec	(hl)		; -1
	ret	p		; ... im Bereich 
	inc	(hl)		; wenn ausserhalb Bildschirm
	inc	hl		; Zeiger auf CURVER
	inc	(hl)		; Test ob Zeile 0
	dec	(hl)
	ret	z		; ... dann Fertig sonst
	dec	hl		; Zeiger auf CURHOR
	ld	(hl),ncol-1	; und letzte Position init
	
cur$up:

	;
	; Cursor eine Zeile nach oben. Kein Downscroll wenn
	; in Zeile 0
	;

	ld	hl,curver	; V-Pos
	dec	(hl)		; -1
	ret	p		; ... wenn im Bereich sonst
	inc	(hl)		; wieder auf 0
	scf			; Marker
	ret
	
set$attr:

	;
	; Attribute setzen
	; Anstelle Underline wird derzeit half density verwendet
	;
 
	call	next		; welches ?
	ld	hl,abits	; Zeiger auf Attribute
	cp	'0'		; Normaldarstellung
	jr	z,seta1
	cp	'2'		; Blink
	jr	z,blk$on		
	cp	'4'		; Invers
	jr	z,inv$on
	cp	'6'		; Invers blinkend
	jr	z,seta2
	cp	'8'		; Normal unterstrichen
	jr	z,dim$on	
	cp	':'		; Blink unterstrichen
	jr	z,seta3
	cp	'<'		; Invers unterstrichen 
	jr	z,seta4
	cp	'>'		; Invers blinkend unterstrichen	
	ret	nz		; sonst nichts
	call	seta3		; Blink und DIM
	jr	inv$on		; und Invers
	
seta1:	ld	a,(hl)
	and 	11110011b	; reset Blink und DIM
	ld	(hl),a
	jr	inv$off		; und Invers

seta2:	set	2,(hl)		; Blink
	jr	inv$on

seta3:	ld	a,(hl)
	or	00001100b	; Blink und DIM
	ld	(hl),a
	ret

seta4:	set	3,(hl)		; DIM	

	;
	; **** INVERS setzen und ruecksetzen
	;

inv$on:	ld	a,80h		; INVERS-Flag ein ( Bit 7 )
	db	06h		; Skip ( LD B,.... )
inv$off:xor	a		; INVERS-Flag aus ( bit 7)
	ld	(invflg),a
	ret

	;
	; **** DIM ( Half-density) setzen und ruecksetzen
	;

dim$on:	set	3,(hl)		; !!!! HL zeigt auf ABITS
	ret
	
dim$off:res	3,(hl)
	ret
	
	;
	; **** Blinken setzen und ruecksetzen
	;

blk$on:	set	2,(hl)
	ret

blk$off:res	2,(hl)
	ret

	;
	; **** Zeichensaetze ein/ausschalten
	;

	;
	; Nachfolgend werden die vier ersten Zeichengeneratoren
	; zugeschaltet. Umschaltung erfolgt durch Initialisierung
	; eines 'anderen' Zeichensatzes
	;

zs$sel:

	call	next		; naechstes Zeichen
	ld	hl,abits	; Zeiger auf Attribute nachladen
	cp	'4'		; freier Zeichensatz
	jr	z,freeon
	cp	'2'		; deutsch ?
	jr	z,dton
	cp	'3'		; Satz 3
	jr	z,gron
	cp	'1'		; Satz 4
	ret	nz

	;
	; Bei Bedarf koennen hier die Zeichensaetze 5..8 eingebunden
	; werden, falls ein entsprechendes EPROM (xxx128) benutzt wird
	;

	
	;
	; **** ASCII ( Normaler 1. Zeichensatz )
	;

ascon:	res	1,(hl)		; Bereich A
	res	4,(hl)		; und NICHT 2.Gruppe
	ret
	
	;
	; **** Deutscher ( 2. Zeichensatz )
	;

dton:	res	1,(hl)		; Bereich A
	set	4,(hl)		; 2.Gruppe
	ret
	
	;
	; **** Grafik ( 3. Zeichensatz )
	;

gron:	set	1,(hl)		; Bereich B
	res	4,(hl)		; 1. Gruppe
	ret
	
	;
	; **** Freier Zeichensatz ( 4. Zeichensatz )
	;

freeon:	set	1,(hl)		; Bereich B
	set	4,(hl)		; 2. Gruppe
	ret
	
	;
	; **** Cursor Attribute setzen
	;

cur$attr:
	
	call	next		; erwarte 0..4
	cp	'4'			
	jr	z,att4		; Cursor Underline
	cp	'3'
	jr	z,att3		; Cursor blinkend Underline
	cp	'2'
	jr	z,att2		; Cursor massiver Block
	cp	'1'
	jr	z,att1		; Cursor blinkender Block
	cp	'0'		; kein Cursor
	ret	nz		; sonst nichts

att0:
	
	;
	; **** kein Cursor ****
	;

	ld	c,10h
	db	21h		; Skip

att1:

	;
	; **** blink Block ****
	; 

	ld	c,60h	
	db	21h

att2:
	
	;
	; **** massiv Block ****
	;

	ld	c,0
	db	21h

att3:	

	;
	; **** Blink Underline ****
	;

	ld	c,69h		; bei kleinem Zeichenzatz!
	db	21h

att4:

	;
	; **** massiv Underline ****
	;

	ld	c,09h		; bei kleinem Zeichensatz
	ld	a,10		; Register 10
	out	(p$vreg),a	; ansprechen
	ld	a,c		; Cursorzuweisung
	out	(p$vfil),a
	ret

	;
	; **** Cursoradressierung 
	;

cur$addr:
	
	;
	; Setzen einer neuen Cursorposition
	; Syntax: ESC = Zeile+20H Spalte+20H
	;

	call	next		; Zeilenadresse holen
	sub	blank		; Offset
	cp	nrow		; gueltig ?
	ret	nc		; ... wenn nicht
	ld	(temp),a	; sonst zwischenspeichern
	call	next		; und Spaltenadresse holen
	sub	blank		; - Offset
	cp	ncol		; im Bereich ?
	ret	nc		; wenn nicht ...
	ld	hl,(temp-1)	; H=Zeile
	ld	l,a		; L=Spalte
	jp	cur$h1		; ablegen und Cursor setzen
	
	;
	; **** Statuszeile 
	;

sline$on:
	
	;
	; Initialisieren der Statuszeile ( INVERS setzen )
	;

	ld	a,80H		; Setze Bit 7
	db	06h		; Skip
	
sline$off:
	
	;
	; Ruecksetzen ( loeschen ) der Statuszeile
	;

	xor	a		; Reset Bit 7
	ld	hl,vram+screen$len ; 1. Position Statuszeile
	ld	d,h		; Kopie nach <DE>
	ld	e,l
	inc	de		; +1
	ld	bc,ncol		; Laenge
	movec
	xor	a		; Reset Zaehler
	ld	(sl$len),a	; Laenge initialisieren
	ret
	
sline$set:
	
	;
	; Beschreiben der Statuszeile bis <CR>
	;

	call	next		; Naechstes Zeichen
	cp	blank		; Ende ?
	jr	nc,slset1	; wenn nicht
	cp	cr		; Ende mit CR
	jr	nz,sline$set	; sonst ignorieren
	xor	a		; Zeilenzaehler
	ld	(sl$len),a	; wieder auf NULL
	ret 
	
slset1:	ld	a,(sl$len)	; Laenge einlesen
	cp	ncol		; noch im Bereich
	jr	nc,sline$set	; wenn nicht, warten auf CR
	inc	a		; naechste Position
	ld	(sl$len),a	; Vorbereiten
	dec	a
	ld	e,a		; in <DE>
	ld	d,0
	ld	hl,vram+screen$len  ; Zeiger auf VRAM
	add	hl,de		; aktuelle Position
	ld	a,80H		; Inversbit
	or	c		; in auszugebendes Zeichen
	storec
	jr	sline$set	; Schleife bis CR
	
rev$lf:

	;
	; Cursor nach 'oben' in oberster Zeile downscroll
	;

	call	cur$up		; Cursor nach oben
	jr	c,scroll$down	; scroll wenn bereits oben
	ret			; sonst fertig

char$ins:

	;
	; Zeile ab Cursorposition nach rechts bis Zeilenende schieben,
	; letztes Zeichen in der Zeile geht verloren. Cursorposition 
	; loeschen
	;

	ld	a,(curver)	; Zeilenposition
	inc	a		; +1
	call	verpos1		; Start naechste Zeile
	dec	hl		; Ende momentaner Zeile
	push	hl		; Rette Zeilenende
	ld	de,(curpos)	; Cursorposition momentan
	or	a		; Reset Carry
	sbc	hl,de		; 
	ld	b,h
	ld	c,l		; Laenge nach BC
	pop	hl		; Zeilenende
	jr	z,chi1		; ... wenn CURPOS=Zeilende
	ld	d,h
	ld	e,l
	dec	hl
	moveu			; transfer
	ex	de,hl
	jr	chi1	

char$del:

	;
	; Zeile wird ab Cursorposition nach rechts bis Zeilenende
	; verschoben. Letztes Zeichen in Zeile entfaellt. Unter Cursor
	; wird ein Blank geschrieben (Zeichen unter Cursor geloescht)
	;

	ld	a,(curver)	; Berechne Start der naechsten Zeile
	inc	a	
	call	verpos1
	dec	hl		; HL=Zeilenende
	push	hl		; retten
	ld	de,(curpos)	; Cursorposition
	push	de		; retten
	or	a		; Reset Carry
	sbc	hl,de		; Zeilenende-Cursorposition
	ld	b,h		; DIFF = Laenge nach BC kopieren
	ld	c,l		; Kopie nach BC
	inc	bc
	pop	hl		; Cursorposition
	pop	de		; Zeilenende
	jr	z,chi1		; wenn am Zeilenende
	ex	de,hl		; Cursorposition nach DE
	push	hl		; Zeilenende retten
	ld	h,d
	ld	l,e		
	inc	hl		; HL=Cursorpos +1
	moves			; Transfer
	pop	hl		; Zeilenende
chi1:	ld	a,blank
	storec
	ret

	;
	; **** Insert & Delete
	;

insert$line:

	;
	; Einfuegen einer Zeile in die Zeile in der sich der Cursor befindet
	; Die folgenden Zeilen werden nach 'unten' gescrollt, die letzte Zeile
	; ist verloren - dannach wird Cursor an Zeilenanfang gesetzt und die
	; 'neue' Zeile geloescht
	;

	call	cur$cr		; an den Zeilenanfang

scroll$down:

	call	verpos		; Zeilenposition berechnen
	ld	(curpos),hl	; ist neue Cursorposition
	ex	de,hl
	ld	hl,screen$end-ncol-1	; Zeiger auf Ende vorletzter Zeile
	push	hl		; retten
	or	a		; Carry=0
	sbc	hl,de		; =Laenge
	pop	de		; Position
	jp	c,clear$eol	; wenn in letzter Zeile
	ld	b,h		; Laenge nach <BC> kopieren
	ld	c,l
	ld	a,b		; Laenge=0 ?
	or	c
	jr	z,iline1
	inc	bc		; Ausgleich
	ld	hl,ncol		; Zeilenlaenge
	add	hl,de		; Berechnen naechste Zeile
	ex	de,hl		; Adresse nach DE
	push	bc		; Alle Werte retten fuer Attribute
	push	de
	push	hl
	moveu
	pop	hl
	pop	de
	ld	bc,-800H	; Offset in ARAM
	add	hl,bc
	ex	de,hl
	add	hl,bc
	ex	de,hl
	pop	bc
	moveu
iline1:	jp	clear$eol	; und jetzt noch Zeile loeschen
	
delete$line:
	
	;
	; loescht die Zeile in der sich der Cursor befindet und scrollt die
	; naechsten Zeilen nach oben. Die letzte Zeile wird geloescht
	;

	call	cur$cr
	call	verpos		; Zeilenposition berechnen
	ld	(curpos),hl	; ist neue Cursorposition
	ex	de,hl
	ld	hl,screen$end-ncol ; ab wo scrollen ?
	or	a		; Reset Carryflag
	sbc	hl,de		; Laenge des scrolls
	jp	c,clear$eol	; wenn in letzter Zeile	
	ld	b,h		; Laenge in <BC> kopieren
	ld	c,l
	ld	a,b		; Teste ob NULL-Laenge
	or	c
	jp	z,clear$eol
	ld	hl,ncol		; Anfang naechster Zeile
	add	hl,de
	push	bc
	push	de
	push	hl		; Retten fuer Attribute
	moves
	push	de		; letzte Adresse des moves
	pop	hl
	inc	de
	ld	bc,ncol-1	; 1 Zeile
	xor	a
	movec			; loeschen
	pop	hl
	pop	de
	ld	bc,-800H
	add	hl,bc
	ex	de,hl
	add	hl,bc
	ex	de,hl
	pop	bc		; Laenge
	moves
	push	de
	pop	hl
	inc	de
	ld	bc,ncol-1
	movec			; loeschen
	ret
	
	;
	; **** Aussprung & Next
	;

next:

	;
	; veranlasst das 'Durchreichen' des naechsten Zeichens
	; an das Unterprogramm das NEXT aufgerufen hat. Das Zeichen
	; wird NICHT als Ausgabezeichen bewertet.
	;

	pop	hl		; Returnadresse des Callers
	ld	(njump),hl	; als Sprungadresse setzen
	pop	hl		; Return auf VRET vom Stack
	db	3eh		; [ LD A,... ]
	
vret:

	;
	; Returnadresse zum BIOS-Caller
	; Setzt Cursor an neue Position und uebergibt
	; alle Register unbeschaedigt und Ausgegebens Zeichen in <A>
	; Flags sind ohne Bedeutung
	;

	xor	a		; Reset NXTFLG
	ld	(nxtflg),a	; A=0 kein NEXT a=NZ NEXT
	
setpos:

	;
	; Setzt Cursorposition entsprechend CURPOS
	;

	call	verpos		; Berechne Zeilenanfang
	ld	a,(curhor)	; H-Pos
	ld	c,a		; Nach <BC>
	ld	b,0
	add	hl,bc
	ld	(curpos),hl	; Neue Position Retten
	ld	a,0fh		; LOW-Cursor Register
	out	(p$vreg),a	; ansprechen
	ld	a,l		; low Byte (CURPOS)
	out	(p$vfil),a	; an Kontroller
	ld	a,0eh		; HIGH-Cursor Register
	out	(p$vreg),a	; ansprechen
	ld	a,h		; high Byte (CURPOS)
	and	00000111b	; Nur 11bit ( 3 Bit high )
	out	(p$vfil),a	; an Kontroller
	pop	hl
	pop	de
	pop	bc
	ret
	
	;
	; ****************
	; * RAM-Bereich  *
	; ****************
	;

temp:	db	0		; Zwischenspeicher
curhor:	db	0		; aktuelle Spaltennummer
curver:	db	0		; aktuelle Zeilennummer
curpos:	dw	0		; aktuelle VRAM-Position
abits:	db	0		; Attributflags
invflg:	db	0		; Inversflag
nxtflg:	db	0		; NEXT-Sequenz wenn NZ
sl$len:	db	0		; Zeilenlaenge Statuszeile



; ****************************************************************
; * Ende des VIDEI-80 Treibers V83.INC				 *
; ****************************************************************
